@@ -1,10 +1,8 @@
-// -*- C++ -*-
-
 /* 
  * Gnome Chemistry Utils
  * element.cc 
  *
- * Copyright (C) 2002-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -24,6 +22,7 @@
 
 #include "config.h"
 #include "element.h"
+#include "xml-utils.h"
 #include <libxml/parser.h>
 #include <libxml/parserInternals.h>
 #include <libxml/xmlmemory.h>
@@ -50,6 +49,15 @@ static void ReadValue (char const *source, GcuValue &value)
 	value.delta = (*buf == '(')? strtol (buf + 1, NULL, 10): 0;
 }
 
+static void ReadDimensionalValue (char const *source, GcuDimensionalValue &value)
+{
+	char *buf, *dot;
+	value.value = strtod (source, &buf);
+	dot = strchr (source, '.');
+	value.prec = (dot)? buf - dot - 1: 0;
+	value.delta = (*buf == '(')? strtol (buf + 1, NULL, 10): 0;
+}
+
 namespace gcu
 {
 
@@ -77,7 +85,7 @@ EltTable Table;
 
 EltTable::EltTable()
 {
-	bindtextdomain (GETTEXT_PACKAGE, DATADIR"/locale");
+	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
 #ifdef ENABLE_NLS
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 #endif
@@ -93,7 +101,7 @@ EltTable::EltTable()
 	Langs["it"] = _("Italian");
 	Langs["pl"] = _("Polish");
 	Langs["ru"] = _("Russian");
-	if (!(xml = xmlParseFile (DATADIR"/gchemutils/elements.xml")))
+	if (!(xml = xmlParseFile (PKGDATADIR"/elements.xml")))
 	{
 		g_error (_("Can't find and read elements.xml"));
 	}
@@ -298,25 +306,21 @@ Element::Element(int Z, const char* Symbol)
 Element::~Element()
 {
 	while (!m_radii.empty()) {
-		GcuAtomicRadius *radius = m_radii.back();
-		if (radius) {
-			if (radius->scale) g_free(radius->scale);
-			delete radius;
-		}
+		delete m_radii.back();
 		m_radii.pop_back();
 	}
 	while (!m_en.empty()) {
-		GcuElectronegativity* en = m_en.back();
-		if (en) {
-			if (en->scale) g_free(en->scale);
-			delete en;
-		}
+		delete m_en.back();
 		m_en.pop_back();
 	}
 	while (!m_isotopes.empty ()) {
 		delete (m_isotopes.back ());
 		m_isotopes.pop_back ();
 	}
+	map<string, Value*>::iterator i, iend = props.end ();
+	for (i = props.begin (); i != iend; i++)
+		delete (*i).second;
+	props.clear ();
 }
 
 const gchar* Element::Symbol(gint Z)
@@ -421,7 +425,7 @@ void Element::LoadRadii ()
 	static bool loaded = false;
 	if (loaded)
 		return;
-	if (!(xml = xmlParseFile (DATADIR"/gchemutils/radii.xml")))
+	if (!(xml = xmlParseFile (PKGDATADIR"/radii.xml")))
 	{
 		g_error (_("Can't find and read radii.xml"));
 	}
@@ -492,7 +496,7 @@ void Element::LoadRadii ()
 						xmlFree (buf);
 					buf = (char*) xmlGetProp (child, (xmlChar*) "value");
 					if (buf) {
-						ReadValue (buf, (GcuValue&) radius->value) ;
+						ReadDimensionalValue (buf, radius->value) ;
 						radius->value.unit = (*it).c_str ();
 						Elt->m_radii.push_back (radius);
 						xmlFree (buf);
@@ -521,7 +525,7 @@ void Element::LoadElectronicProps ()
 	static bool loaded = false;
 	if (loaded)
 		return;
-	if (!(xml = xmlParseFile (DATADIR"/gchemutils/elecprops.xml")))
+	if (!(xml = xmlParseFile (PKGDATADIR"/elecprops.xml")))
 	{
 		g_error (_("Can't find and read elecprops.xml"));
 	}
@@ -547,7 +551,7 @@ void Element::LoadElectronicProps ()
 					en->Z = Z;	//FIXME: is it really useful there?
 					buf = (char*) xmlGetProp (child, (xmlChar*) "scale");
 					if (buf) {
-						en->scale = g_strdup (buf);
+						en->scale = GetStaticScale (buf);
 						xmlFree (buf);
 					} else
 						en->scale = NULL;
@@ -679,7 +683,7 @@ void Element::LoadIsotopes ()
 	static bool loaded = false;
 	if (loaded)
 		return;
-	if (!(xml = xmlParseFile (DATADIR"/gchemutils/isotopes.xml")))
+	if (!(xml = xmlParseFile (PKGDATADIR"/isotopes.xml")))
 	{
 		g_error (_("Can't find and read isotopes.xml"));
 	}
@@ -765,6 +769,7 @@ void Element::LoadAllData ()
 	LoadRadii ();
 	LoadElectronicProps ();
 	LoadIsotopes ();
+	LoadBODR ();
 }
 
 IsotopicPattern *Element::GetIsotopicPattern (unsigned natoms)
@@ -816,3 +821,125 @@ GcuDimensionalValue const *Element::GetElectronAffinity (unsigned rank)
 {
 	return (rank <= m_ae.size ())? &m_ae[rank - 1]: NULL;
 }
+
+void Element::LoadBODR ()
+{
+	char *old_num_locale;
+	old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
+	setlocale (LC_NUMERIC, "C");
+	xmlDocPtr xml = xmlParseFile (BODR_PKGDATADIR"/elements.xml");
+	if (!xml)
+		return;
+	xmlNodePtr node = xml->children, child;
+	char *buf = NULL, *unit;
+	Element *elt;
+	map <string, Value*> props;
+	if (!strcmp ((char const*) node->name, "list")) {
+		node = node->children;
+		while (node) {
+			if (!strcmp ((char const*) node->name, "atom")) {
+				elt = NULL;
+				child = node->children;
+				while (child) {
+					if (!strcmp ((char const*) child->name, "scalar")) {
+						Value *val;
+						buf = (char*) xmlGetProp (child, (xmlChar const*) "dataType");
+						if (!strcmp (buf, "xsd:float")) {
+							char *end, *dot;
+							xmlFree (buf);
+							buf = (char*) xmlNodeGetContent (child);
+							double x = strtod (buf, &end);
+							dot = strchr (buf, '.');
+							int prec = (dot)? end - dot - 1: 0;
+							unit = (char*) xmlGetProp (child, (xmlChar const*) "units");
+							xmlFree (buf);
+							buf = (char*) xmlGetProp (child, (xmlChar const*) "errorValue");
+							int delta;
+							if (buf)
+								delta = strtol (buf, NULL, 10);
+							else
+								delta = 0;
+							xmlFree (buf);
+							if (unit) {
+								DimensionalValue *v = new DimensionalValue ();
+								if (!strcmp (unit, "units:atmass"))
+									// amu stands for "atomic mass units"
+									v->val.unit = _("amu");
+								else if (!strcmp (unit, "units:ev"))
+									v->val.unit = "eV";
+								else if (!strcmp (unit, "units:ang"))
+									v->val.unit = "Å";
+								else if (!strcmp (unit, "siUnits:kelvin"))
+									v->val.unit = "K";
+//								else if (!strcmp (unit, ""))
+//									v->val.unit = "";
+								else
+									v->val.unit = "";
+								xmlFree (unit);
+								v->val.value = x;
+								v->val.prec = prec;
+								v->val.delta = delta;
+								val = v;
+							} else {
+								SimpleValue *v = new SimpleValue ();
+								v->val.value = x;
+								v->val.prec = prec;
+								v->val.delta = delta;
+								val = v;
+							}
+							buf = (char*) xmlGetProp (child, (xmlChar const*) "dictRef");
+							if (elt)
+								elt->props[(strncmp (buf, "bo:", 3))? buf: buf + 3] = val;
+							xmlFree (buf);
+						} else if (!strcmp (buf, "xsd:String") || !strcmp (buf, "xsd:string")) {
+							buf = (char*) xmlGetProp (child, (xmlChar const*) "dictRef");
+							char *val = (char*) xmlNodeGetContent (child);
+							if (elt)
+								elt->sprops[(strncmp (buf, "bo:", 3))? buf: buf + 3] = val;
+							xmlFree (buf);
+							xmlFree (val);
+						} else if (!strcmp (buf, "xsd:int") || !strcmp (buf, "xsd:Integer")) {
+							xmlFree (buf);
+							buf = (char*) xmlNodeGetContent (child);
+							int val = strtol (buf, NULL, 10);
+							xmlFree (buf);
+							buf = (char*) xmlGetProp (child, (xmlChar const*) "dictRef");
+							if (!strcmp (buf, "bo:atomicNumber"))
+								elt = Table[val];
+							else if (elt)
+								elt->iprops[(strncmp (buf, "bo:", 3))? buf: buf + 3] = val;
+							xmlFree (buf);
+						} else if (!strcmp (buf, "xsd:date")) {
+							// assumeing only the discovery year is useful for us
+							xmlFree (buf);
+							buf = (char*) xmlNodeGetContent (child);
+							int val = strtol (buf, NULL, 10);
+							xmlFree (buf);
+							buf = (char*) xmlGetProp (child, (xmlChar const*) "dictRef");
+							if (!strcmp (buf, "bo:discoveryDate"))
+								elt->iprops["discoveryDate"] = val;
+							xmlFree (buf);
+						}
+					}
+					if (props.size () > 0) {
+						map <string, Value*>:: iterator i, iend = props.end ();
+						for (i = props.begin (); i != iend; i++)
+							elt->props[(*i).first] = (*i).second;
+						props.clear ();
+					}
+					child = child->next;
+				}
+			}
+			node = node->next;
+		}
+	}
+	xmlFreeDoc (xml);
+	setlocale (LC_NUMERIC, old_num_locale);
+	g_free (old_num_locale);
+}
+
+int Element::GetIntegerProperty (char const *property_name)
+{
+	map<string, int>::iterator i = iprops.find (property_name);
+	return (i == iprops.end ())? GCU_ERROR: (*i).second;
+}
