@@ -40,7 +40,7 @@
 
 static unsigned short nNewDocs = 1;
 
-gcApplication::gcApplication(): Application ("gcrystal-unstable")
+gcApplication::gcApplication(): Application ("gcrystal")
 {
 }
 
